@@ -6,12 +6,10 @@ from flask import Flask
 from sqlalchemy import create_engine, event
 from sqlalchemy.engine import Engine
 from sqlalchemy.engine import ExceptionContext
+from sqlalchemy.exc import OperationalError
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.orm import sessionmaker
 
-from ..log import flaskz_logger
-from ..utils import Attribute
-
 DBSession = sessionmaker(autocommit=False)
 
 ModelBase = declarative_base()
@@ -19,14 +17,14 @@ ModelBase = declarative_base()
 
 def init_model(app):
     """
-    Initialize the database
+    Initialize the database.
+
+    Example:
+        init_model(flask_app)
+
     """
     is_app = isinstance(app, Flask)
-    if is_app:
-        app_config = app.config
-    else:
-        app_config = app
-    # app_config = app.config
+    app_config = get_app_config_items(app) or {}
     database_uri = app_config.get('FLASKZ_DATABASE_URI')
 
     # enable sqlite foreign key
@@ -51,7 +49,8 @@ def init_model(app):
                 engine_kwargs[engine_key] = app_config.get(config_key)
 
         engine = create_engine(database_uri, **engine_kwargs)
-        DBSession.configure(binds={ModelBase: engine})  # for multiple db
+        session_kwargs = app_config.get('FLASKZ_DATABASE_SESSION_KWARGS') or {}  # @2023-08-25: add FLASKZ_DATABASE_SESSION_KWARGS config
+        DBSession.configure(binds={ModelBase: engine}, **session_kwargs)  # for multiple db
 
         with engine.connect():  # connect test
             flaskz_logger.info('Database ' + database_uri + ' is ready\n')
@@ -61,7 +60,10 @@ def init_model(app):
 
         @event.listens_for(engine, "handle_error")
         def handle_engine_error(context: ExceptionContext):  # @2023-02-01: add error handler
-            flaskz_logger.error('Engine error:\n' + str(context.original_exception))
+            original_exception = getattr(context, 'original_exception', None)
+            flaskz_logger.error('Engine error:\n' + str(original_exception))
+            if not isinstance(original_exception, OperationalError):  # @2023-08-25: add
+                return
             if engine_kwargs.get('pool_pre_ping') is not True:
                 if not context.connection or context.sqlalchemy_exception.connection_invalidated:
                     now = datetime.now().timestamp()
@@ -70,7 +72,7 @@ def init_model(app):
                         engine_err_info['connect_time'] = now
                         engine.connect()  # reconnect
 
-    except Exception as e:
+    except Exception:
         flaskz_logger.exception('Connect to database ' + database_uri + ' error\n')
         return
 
@@ -104,9 +106,6 @@ def init_model(app):
                 'context': context,
             }))
 
-        # if duration > db_slow_time:
-        #     app_logger.warning('Slow query: %s\nParameters: %s\nDuration: %fms\n' % (statement, parameters, duration))
-
         db_slow_time = app_config.get('FLASKZ_DATABASE_DEBUG_SLOW_TIME', -1)  # set config['FLASKZ_DATABASE_DEBUG_SLOW_TIME'] to enable
         db_access_times = app_config.get('FLASKZ_DATABASE_DEBUG_ACCESS_TIMES', -1)  # set config['FLASKZ_DATABASE_DEBUG_ACCESS_TIMES'] to enable
         if db_slow_time > 0 or db_access_times > 0:
@@ -131,3 +130,7 @@ def init_model(app):
 from ._base import *
 from ._model import *
 from ._util import *
+from ._query_util import *
+from ..utils._cls import Attribute
+from ..utils._app import get_app_config_items
+from ..log import flaskz_logger
