@@ -12,20 +12,21 @@ __all__ = ['get_current_model_rest_manager_callback',
 
 
 def get_current_model_rest_manager_callback(callback_name):
-    model_rest_manager = getattr(current_app, 'model_rest_manager', None)
-    if model_rest_manager:
-        return getattr(model_rest_manager, callback_name)
+    if current_app:
+        model_rest_manager = getattr(current_app, 'model_rest_manager', None)
+        if model_rest_manager:
+            return getattr(model_rest_manager, callback_name, None)
 
 
 def rest_login_required():
     """
-     If you decorate a view with this, it will ensure that the current user is
-    logged in and authenticated before calling the actual view.
+    If you decorate a view with this, it will ensure that the current user is logged in and authenticated before calling the actual view.
 
-    @sys_mgmt_bp.route('/auth/account/', methods=['GET', 'POST'])
-    @rest_login_required()
-    def sys_auth_account_query():
-        pass
+    Example:
+        @sys_mgmt_bp.route('/auth/account/', methods=['GET', 'POST'])
+        @rest_login_required()
+        def sys_auth_account_query():
+            pass
 
     :return:
     """
@@ -46,23 +47,24 @@ def rest_login_required():
     return decorate
 
 
-def rest_permission_required(module, op_permission=None):
+def rest_permission_required(module, action=None):  # @2023-10-10, op_permission--> action
     """
-    If you decorate a view with this, it will ensure that the current user
-    has the module permission and operation permission before calling the actual view.
+    If you decorate a view with this, it will ensure that the current user.
+    has the module permission and action permission before calling the actual view.
 
-    @sys_mgmt_bp.route('/role/', methods=['GET'])
-    @rest_permission_required('role')
-    def sys_role_query():
-        pass
+    Example:
+        @sys_mgmt_bp.route('/role/', methods=['GET'])
+        @rest_permission_required('role')
+        def sys_role_query():
+            pass
 
-    @sys_mgmt_bp.route('/role/', methods=['POST'])
-    @rest_permission_required('role', 'add')
-    def sys_role_add():
-        pass
+        @sys_mgmt_bp.route('/role/', methods=['POST'])
+        @rest_permission_required('role', 'add')
+        def sys_role_add():
+            pass
 
-    :param module:
-    :param op_permission:
+    :param module: module permission to be checked
+    :param action: action permission to be checked
     :return:
     """
 
@@ -72,7 +74,7 @@ def rest_permission_required(module, op_permission=None):
             check_result = True
             permission_check_callback = get_current_model_rest_manager_callback('permission_check_callback')
             if permission_check_callback:
-                check_result = permission_check_callback(module, op_permission)
+                check_result = permission_check_callback(module, action)
             if check_result is not True:
                 return check_result
             return func(*args, **kwargs)
@@ -85,6 +87,7 @@ def rest_permission_required(module, op_permission=None):
 def get_rest_log_msg(api_info, req_data, success, res_data):
     """
     Return the log message text.
+
     :param api_info:
     :param req_data:
     :param success:
@@ -100,7 +103,11 @@ def get_rest_log_msg(api_info, req_data, success, res_data):
 
 def log_operation(*args, **kwargs):
     """
-    Log operation
+    Log operation/action.
+
+    Example:
+        log_operation('users', 'login', success, request_json.get('username'), res_data)
+
     :param args:
     :param kwargs:
     :return:
@@ -110,11 +117,13 @@ def log_operation(*args, **kwargs):
         logging_callback(*args, **kwargs)
 
 
-def gen_route_method(method, url_prefix):
+def gen_route_method(method, url_prefix, view_func_name=None):
     """
-    Generate endpoint unique function name
+    Generate endpoint unique function name.
+
     :param method:
     :param url_prefix:
+    :param view_func_name:
     :return:
     """
 
@@ -122,10 +131,13 @@ def gen_route_method(method, url_prefix):
         def wrap(*args, **kwargs):
             return f(*args, **kwargs)
 
-        methods = url_prefix.split('/')
-        methods.append(method)
-        methods.insert(0, 'model_rest')
-        wrap.__name__ = '_'.join(filter_list(methods, lambda item: item != ''))
+        if view_func_name is None:
+            methods = url_prefix.split('/')
+            methods.append(method)
+            methods.insert(0, 'model_route')
+            wrap.__name__ = '_'.join(filter_list(methods, lambda item: item != '')).replace('-', '_')
+        else:
+            wrap.__name__ = view_func_name
         return wrap
 
     return decorator
