@@ -1,6 +1,7 @@
+from datetime import datetime
 from threading import Timer
 
-__all__ = ['set_timeout', 'set_interval']
+__all__ = ['set_interval', 'set_timeout', 'run_at']
 
 
 class _IntervalTimer(object):
@@ -42,17 +43,18 @@ def set_interval(interval, function, args=None, kwargs=None, immediately=False,
     Execute a function periodically at a specified time interval(milliseconds).
     The result of the function execution is False, then the loop is interrupted.
 
-        t = set_interval(10, print, ('Hello, World!',))
-        # t.cancel()    # Stop the timer
-
     .. versionadded:: 1.0
 
-    :param daemon: if true, the timer will be daemon(Daemon threads are abruptly stopped at shutdown)
+    Example:
+        t = set_interval(10000, print, ('Hello, World!',)) # 10s
+        # t.cancel()    # Stop the timer
+
     :param interval: time interval in milliseconds
     :param function: callback function, which should do exception catching*
     :param args:
     :param kwargs:
     :param immediately: If True, the function will be executed immediately, otherwise it will be executed after the interval time
+    :param daemon: if true, the timer will be daemon(Daemon threads are abruptly stopped at shutdown)
     :return:
     """
     t = _IntervalTimer(interval, function, args=args, kwargs=kwargs, immediately=immediately, daemon=daemon)
@@ -63,19 +65,57 @@ def set_timeout(interval, function, args=None, kwargs=None, daemon=True):
     """
     Set a timer which executes a function once the timer expires(milliseconds).
 
-        t = set_timeout(10, print, ('Hello, World!',))
-        # t.cancel()    # Stop the timer
-
      .. versionadded:: 1.0
 
-    :param daemon: if true, the timer will be daemon(Daemon threads are abruptly stopped at shutdown)
+    Example:
+        t = set_timeout(10000, print, ('Hello, World!',)) # 10s
+        # t.cancel()    # Stop the timer
+
     :param interval: delay time in milliseconds before the specified function is executed
-    :param function:
-    :param args:
-    :param kwargs:
+    :param function: the function to be executed
+    :param args: the args of the function
+    :param kwargs: the kwargs of the function
+    :param daemon: whether the timer is a daemon thread, default True(Daemon thread stops abruptly when main thread is shut down)
     :return:
     """
     t = Timer(interval / 1000, function, args=args, kwargs=kwargs)
     t.daemon = daemon
     t.start()
     return t
+
+
+def run_at(at_time, function, args=None, kwargs=None, daemon=True, time_format='%Y-%m-%d %H:%M:%S'):
+    """
+    Call a function at the specified datetime.
+
+    .. versionadded:: 1.6.1
+
+    Example:
+        t1 = run_at('2023-06-28 14:53:00', print, ['run_at'])   # time str
+        # t1.cancel()    # Stop the timer
+
+        t2 = run_at(1629811200, print, ['run_at'])  # timestamp
+        # t2.cancel()    # Stop the timer
+
+    :param at_time: the specified datetime(str/timestamp)
+    :param function: the function to be executed
+    :param args: the args of the function
+    :param kwargs: the kwargs of the function
+    :param daemon: whether the timer is a daemon thread, default True(Daemon thread stops abruptly when main thread is shut down)
+    :param time_format: the format of the specified datetime
+    :return:
+    """
+    time_type = type(at_time)
+    if time_type is str:
+        at_time = datetime.strptime(at_time, time_format)
+    elif time_type is int:
+        at_time = datetime.fromtimestamp(at_time)
+    now = datetime.now()
+    timeout_ms = (at_time - now).total_seconds()
+    if timeout_ms < 0:
+        return False
+    elif timeout_ms == 0:
+        function(*(args if args is not None else []), **(kwargs if kwargs is not None else {}))
+        return
+
+    return set_timeout(timeout_ms * 1000, function, args=args, kwargs=kwargs, daemon=daemon)
